import { GetFloorChildDto } from '../dtos/get.floor.dto';
import { SpaceTypeRepository } from '../../../libs/common/src/modules/space-type/repositories/space.type.repository';
import {
  Injectable,
  HttpException,
  HttpStatus,
  BadRequestException,
} from '@nestjs/common';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import { AddFloorDto, AddUserFloorDto } from '../dtos';
import {
  FloorChildInterface,
  FloorParentInterface,
  GetFloorByUserUuidInterface,
  GetFloorByUuidInterface,
  RenameFloorByUuidInterface,
} from '../interface/floor.interface';
import { SpaceEntity } from '@app/common/modules/space/entities';
import { UpdateFloorNameDto } from '../dtos/update.floor.dto';
import { UserSpaceRepository } from '@app/common/modules/user-space/repositories';

@Injectable()
export class FloorService {
  constructor(
    private readonly spaceRepository: SpaceRepository,
    private readonly spaceTypeRepository: SpaceTypeRepository,
    private readonly userSpaceRepository: UserSpaceRepository,
  ) {}

  async addFloor(addFloorDto: AddFloorDto) {
    try {
      const spaceType = await this.spaceTypeRepository.findOne({
        where: {
          type: 'floor',
        },
      });

      const floor = await this.spaceRepository.save({
        spaceName: addFloorDto.floorName,
        parent: { uuid: addFloorDto.buildingUuid },
        spaceType: { uuid: spaceType.uuid },
      });
      return floor;
    } catch (err) {
      throw new HttpException(err.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }

  async getFloorByUuid(floorUuid: string): Promise<GetFloorByUuidInterface> {
    try {
      const floor = await this.spaceRepository.findOne({
        where: {
          uuid: floorUuid,
          spaceType: {
            type: 'floor',
          },
        },
        relations: ['spaceType'],
      });
      if (!floor || !floor.spaceType || floor.spaceType.type !== 'floor') {
        throw new BadRequestException('Invalid floor UUID');
      }

      return {
        uuid: floor.uuid,
        createdAt: floor.createdAt,
        updatedAt: floor.updatedAt,
        name: floor.spaceName,
        type: floor.spaceType.type,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Floor not found', HttpStatus.NOT_FOUND);
      }
    }
  }
  async getFloorChildByUuid(
    floorUuid: string,
    getFloorChildDto: GetFloorChildDto,
  ): Promise<FloorChildInterface> {
    try {
      const { includeSubSpaces, page, pageSize } = getFloorChildDto;

      const space = await this.spaceRepository.findOneOrFail({
        where: { uuid: floorUuid },
        relations: ['children', 'spaceType'],
      });

      if (!space || !space.spaceType || space.spaceType.type !== 'floor') {
        throw new BadRequestException('Invalid floor UUID');
      }
      const totalCount = await this.spaceRepository.count({
        where: { parent: { uuid: space.uuid } },
      });

      const children = await this.buildHierarchy(
        space,
        includeSubSpaces,
        page,
        pageSize,
      );

      return {
        uuid: space.uuid,
        name: space.spaceName,
        type: space.spaceType.type,
        totalCount,
        children,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Floor not found', HttpStatus.NOT_FOUND);
      }
    }
  }

  private async buildHierarchy(
    space: SpaceEntity,
    includeSubSpaces: boolean,
    page: number,
    pageSize: number,
  ): Promise<FloorChildInterface[]> {
    const children = await this.spaceRepository.find({
      where: { parent: { uuid: space.uuid } },
      relations: ['spaceType'],
      skip: (page - 1) * pageSize,
      take: pageSize,
    });

    if (!children || children.length === 0 || !includeSubSpaces) {
      return children
        .filter(
          (child) =>
            child.spaceType.type !== 'floor' &&
            child.spaceType.type !== 'building' &&
            child.spaceType.type !== 'community',
        ) // Filter remaining floor and building and community types
        .map((child) => ({
          uuid: child.uuid,
          name: child.spaceName,
          type: child.spaceType.type,
        }));
    }

    const childHierarchies = await Promise.all(
      children
        .filter(
          (child) =>
            child.spaceType.type !== 'floor' &&
            child.spaceType.type !== 'building' &&
            child.spaceType.type !== 'community',
        ) // Filter remaining floor and building and community types
        .map(async (child) => ({
          uuid: child.uuid,
          name: child.spaceName,
          type: child.spaceType.type,
          children: await this.buildHierarchy(child, true, 1, pageSize),
        })),
    );

    return childHierarchies;
  }

  async getFloorParentByUuid(floorUuid: string): Promise<FloorParentInterface> {
    try {
      const floor = await this.spaceRepository.findOne({
        where: {
          uuid: floorUuid,
          spaceType: {
            type: 'floor',
          },
        },
        relations: ['spaceType', 'parent', 'parent.spaceType'],
      });
      if (!floor || !floor.spaceType || floor.spaceType.type !== 'floor') {
        throw new BadRequestException('Invalid floor UUID');
      }

      return {
        uuid: floor.uuid,
        name: floor.spaceName,
        type: floor.spaceType.type,
        parent: {
          uuid: floor.parent.uuid,
          name: floor.parent.spaceName,
          type: floor.parent.spaceType.type,
        },
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Floor not found', HttpStatus.NOT_FOUND);
      }
    }
  }

  async getFloorsByUserId(
    userUuid: string,
  ): Promise<GetFloorByUserUuidInterface[]> {
    try {
      const floors = await this.userSpaceRepository.find({
        relations: ['space', 'space.spaceType'],
        where: {
          user: { uuid: userUuid },
          space: { spaceType: { type: 'floor' } },
        },
      });

      if (floors.length === 0) {
        throw new HttpException(
          'this user has no floors',
          HttpStatus.NOT_FOUND,
        );
      }
      const spaces = floors.map((floor) => ({
        uuid: floor.space.uuid,
        name: floor.space.spaceName,
        type: floor.space.spaceType.type,
      }));

      return spaces;
    } catch (err) {
      if (err instanceof HttpException) {
        throw err;
      } else {
        throw new HttpException('user not found', HttpStatus.NOT_FOUND);
      }
    }
  }
  async addUserFloor(addUserFloorDto: AddUserFloorDto) {
    try {
      await this.userSpaceRepository.save({
        user: { uuid: addUserFloorDto.userUuid },
        space: { uuid: addUserFloorDto.floorUuid },
      });
    } catch (err) {
      if (err.code === '23505') {
        throw new HttpException(
          'User already belongs to this floor',
          HttpStatus.BAD_REQUEST,
        );
      }
      throw new HttpException(
        err.message || 'Internal Server Error',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async renameFloorByUuid(
    floorUuid: string,
    updateFloorDto: UpdateFloorNameDto,
  ): Promise<RenameFloorByUuidInterface> {
    try {
      const floor = await this.spaceRepository.findOneOrFail({
        where: { uuid: floorUuid },
        relations: ['spaceType'],
      });

      if (!floor || !floor.spaceType || floor.spaceType.type !== 'floor') {
        throw new BadRequestException('Invalid floor UUID');
      }

      await this.spaceRepository.update(
        { uuid: floorUuid },
        { spaceName: updateFloorDto.floorName },
      );

      // Fetch the updated floor
      const updatedFloor = await this.spaceRepository.findOneOrFail({
        where: { uuid: floorUuid },
        relations: ['spaceType'],
      });

      return {
        uuid: updatedFloor.uuid,
        name: updatedFloor.spaceName,
        type: updatedFloor.spaceType.type,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Floor not found', HttpStatus.NOT_FOUND);
      }
    }
  }
}
