import {
  CanActivate,
  ExecutionContext,
  Injectable,
  HttpStatus,
} from '@nestjs/common';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import { BadRequestException } from '@nestjs/common';

@Injectable()
export class CheckCommunityTypeGuard implements CanActivate {
  constructor(private readonly spaceRepository: SpaceRepository) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const req = context.switchToHttp().getRequest();

    try {
      const { buildingName, communityUuid } = req.body;

      if (!buildingName) {
        throw new BadRequestException('buildingName is required');
      }

      if (!communityUuid) {
        throw new BadRequestException('communityUuid is required');
      }

      await this.checkCommunityIsCommunityType(communityUuid);

      return true;
    } catch (error) {
      this.handleGuardError(error, context);
      return false;
    }
  }

  private async checkCommunityIsCommunityType(communityUuid: string) {
    const communityData = await this.spaceRepository.findOne({
      where: { uuid: communityUuid },
      relations: ['spaceType'],
    });

    if (
      !communityData ||
      !communityData.spaceType ||
      communityData.spaceType.type !== 'community'
    ) {
      throw new BadRequestException('Invalid community UUID');
    }
  }

  private handleGuardError(error: Error, context: ExecutionContext) {
    const response = context.switchToHttp().getResponse();
    console.error(error);

    if (error instanceof BadRequestException) {
      response
        .status(HttpStatus.BAD_REQUEST)
        .json({ statusCode: HttpStatus.BAD_REQUEST, message: error.message });
    } else {
      response.status(HttpStatus.NOT_FOUND).json({
        statusCode: HttpStatus.NOT_FOUND,
        message: 'Community not found',
      });
    }
  }
}
