import { RoomService } from '../services/room.service';
import {
  Body,
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Post,
  Put,
  UseGuards,
} from '@nestjs/common';
import { ApiTags, ApiBearerAuth } from '@nestjs/swagger';
import { AddRoomDto, AddUserRoomDto } from '../dtos/add.room.dto';
import { UpdateRoomNameDto } from '../dtos/update.room.dto';
import { CheckUnitTypeGuard } from 'src/guards/unit.type.guard';
import { CheckUserRoomGuard } from 'src/guards/user.room.guard';
import { AdminRoleGuard } from 'src/guards/admin.role.guard';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { RoomPermissionGuard } from 'src/guards/room.permission.guard';

@ApiTags('Room Module')
@Controller({
  version: '1',
  path: 'room',
})
export class RoomController {
  constructor(private readonly roomService: RoomService) {}

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, CheckUnitTypeGuard)
  @Post()
  async addRoom(@Body() addRoomDto: AddRoomDto) {
    try {
      const room = await this.roomService.addRoom(addRoomDto);
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'Room added successfully',
        data: room,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, RoomPermissionGuard)
  @Get(':roomUuid')
  async getRoomByUuid(@Param('roomUuid') roomUuid: string) {
    try {
      const room = await this.roomService.getRoomByUuid(roomUuid);
      return room;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, RoomPermissionGuard)
  @Get('parent/:roomUuid')
  async getRoomParentByUuid(@Param('roomUuid') roomUuid: string) {
    try {
      const room = await this.roomService.getRoomParentByUuid(roomUuid);
      return room;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard, CheckUserRoomGuard)
  @Post('user')
  async addUserRoom(@Body() addUserRoomDto: AddUserRoomDto) {
    try {
      await this.roomService.addUserRoom(addUserRoomDto);
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'user room added successfully',
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('user/:userUuid')
  async getRoomsByUserId(@Param('userUuid') userUuid: string) {
    try {
      return await this.roomService.getRoomsByUserId(userUuid);
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, RoomPermissionGuard)
  @Put('rename/:roomUuid')
  async renameRoomByUuid(
    @Param('roomUuid') roomUuid: string,
    @Body() updateRoomNameDto: UpdateRoomNameDto,
  ) {
    try {
      const room = await this.roomService.renameRoomByUuid(
        roomUuid,
        updateRoomNameDto,
      );
      return room;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
