import { SpaceTypeRepository } from '../../../libs/common/src/modules/space-type/repositories/space.type.repository';
import {
  Injectable,
  HttpException,
  HttpStatus,
  BadRequestException,
} from '@nestjs/common';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import { AddRoomDto, AddUserRoomDto } from '../dtos';
import {
  RoomParentInterface,
  GetRoomByUuidInterface,
  RenameRoomByUuidInterface,
  GetRoomByUserUuidInterface,
} from '../interface/room.interface';
import { UpdateRoomNameDto } from '../dtos/update.room.dto';
import { UserSpaceRepository } from '@app/common/modules/user-space/repositories';

@Injectable()
export class RoomService {
  constructor(
    private readonly spaceRepository: SpaceRepository,
    private readonly spaceTypeRepository: SpaceTypeRepository,
    private readonly userSpaceRepository: UserSpaceRepository,
  ) {}

  async addRoom(addRoomDto: AddRoomDto) {
    try {
      const spaceType = await this.spaceTypeRepository.findOne({
        where: {
          type: 'room',
        },
      });

      const room = await this.spaceRepository.save({
        spaceName: addRoomDto.roomName,
        parent: { uuid: addRoomDto.unitUuid },
        spaceType: { uuid: spaceType.uuid },
      });
      return room;
    } catch (err) {
      throw new HttpException(err.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }

  async getRoomByUuid(roomUuid: string): Promise<GetRoomByUuidInterface> {
    try {
      const room = await this.spaceRepository.findOne({
        where: {
          uuid: roomUuid,
          spaceType: {
            type: 'room',
          },
        },
        relations: ['spaceType'],
      });
      if (!room || !room.spaceType || room.spaceType.type !== 'room') {
        throw new BadRequestException('Invalid room UUID');
      }

      return {
        uuid: room.uuid,
        createdAt: room.createdAt,
        updatedAt: room.updatedAt,
        name: room.spaceName,
        type: room.spaceType.type,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Room not found', HttpStatus.NOT_FOUND);
      }
    }
  }

  async getRoomParentByUuid(roomUuid: string): Promise<RoomParentInterface> {
    try {
      const room = await this.spaceRepository.findOne({
        where: {
          uuid: roomUuid,
          spaceType: {
            type: 'room',
          },
        },
        relations: ['spaceType', 'parent', 'parent.spaceType'],
      });
      if (!room || !room.spaceType || room.spaceType.type !== 'room') {
        throw new BadRequestException('Invalid room UUID');
      }

      return {
        uuid: room.uuid,
        name: room.spaceName,
        type: room.spaceType.type,
        parent: {
          uuid: room.parent.uuid,
          name: room.parent.spaceName,
          type: room.parent.spaceType.type,
        },
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Room not found', HttpStatus.NOT_FOUND);
      }
    }
  }

  async getRoomsByUserId(
    userUuid: string,
  ): Promise<GetRoomByUserUuidInterface[]> {
    try {
      const rooms = await this.userSpaceRepository.find({
        relations: ['space', 'space.spaceType'],
        where: {
          user: { uuid: userUuid },
          space: { spaceType: { type: 'room' } },
        },
      });

      if (rooms.length === 0) {
        throw new HttpException('this user has no rooms', HttpStatus.NOT_FOUND);
      }
      const spaces = rooms.map((room) => ({
        uuid: room.space.uuid,
        name: room.space.spaceName,
        type: room.space.spaceType.type,
      }));

      return spaces;
    } catch (err) {
      if (err instanceof HttpException) {
        throw err;
      } else {
        throw new HttpException('user not found', HttpStatus.NOT_FOUND);
      }
    }
  }
  async addUserRoom(addUserRoomDto: AddUserRoomDto) {
    try {
      await this.userSpaceRepository.save({
        user: { uuid: addUserRoomDto.userUuid },
        space: { uuid: addUserRoomDto.roomUuid },
      });
    } catch (err) {
      if (err.code === '23505') {
        throw new HttpException(
          'User already belongs to this room',
          HttpStatus.BAD_REQUEST,
        );
      }
      throw new HttpException(
        err.message || 'Internal Server Error',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async renameRoomByUuid(
    roomUuid: string,
    updateRoomNameDto: UpdateRoomNameDto,
  ): Promise<RenameRoomByUuidInterface> {
    try {
      const room = await this.spaceRepository.findOneOrFail({
        where: { uuid: roomUuid },
        relations: ['spaceType'],
      });

      if (!room || !room.spaceType || room.spaceType.type !== 'room') {
        throw new BadRequestException('Invalid room UUID');
      }

      await this.spaceRepository.update(
        { uuid: roomUuid },
        { spaceName: updateRoomNameDto.roomName },
      );

      // Fetch the updated room
      const updateRoom = await this.spaceRepository.findOneOrFail({
        where: { uuid: roomUuid },
        relations: ['spaceType'],
      });

      return {
        uuid: updateRoom.uuid,
        name: updateRoom.spaceName,
        type: updateRoom.spaceType.type,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Room not found', HttpStatus.NOT_FOUND);
      }
    }
  }
}
