import {
  Body,
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Put,
  UseGuards,
} from '@nestjs/common';
import { UserService } from '../services/user.service';
import { ApiTags, ApiBearerAuth } from '@nestjs/swagger';
import { JwtAuthGuard } from '../../../libs/common/src/guards/jwt.auth.guard';
import {
  UpdateNameDto,
  UpdateProfilePictureDataDto,
  UpdateRegionDataDto,
  UpdateTimezoneDataDto,
} from '../dtos';
import { CheckProfilePictureGuard } from 'src/guards/profile.picture.guard';

@ApiTags('User Module')
@Controller({
  version: '1',
  path: 'user',
})
export class UserController {
  constructor(private readonly userService: UserService) {}

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get(':userUuid')
  async getUserDetailsByUserUuid(@Param('userUuid') userUuid: string) {
    try {
      return await this.userService.getUserDetailsByUserUuid(userUuid);
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, CheckProfilePictureGuard)
  @Put('/profile-picture/:userUuid')
  async updateProfilePictureByUserUuid(
    @Param('userUuid') userUuid: string,
    @Body() updateProfilePictureDataDto: UpdateProfilePictureDataDto,
  ) {
    try {
      const userData = await this.userService.updateProfilePictureByUserUuid(
        userUuid,
        updateProfilePictureDataDto,
      );
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'profile picture updated successfully',
        data: userData,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put('/region/:userUuid')
  async updateRegionByUserUuid(
    @Param('userUuid') userUuid: string,
    @Body() updateRegionDataDto: UpdateRegionDataDto,
  ) {
    try {
      const userData = await this.userService.updateRegionByUserUuid(
        userUuid,
        updateRegionDataDto,
      );
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'region updated successfully',
        data: userData,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put('/timezone/:userUuid')
  async updateNameByUserUuid(
    @Param('userUuid') userUuid: string,
    @Body() updateTimezoneDataDto: UpdateTimezoneDataDto,
  ) {
    try {
      const userData = await this.userService.updateTimezoneByUserUuid(
        userUuid,
        updateTimezoneDataDto,
      );
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'timezone updated successfully',
        data: userData,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put('/name/:userUuid')
  async updateTimezoneByUserUuid(
    @Param('userUuid') userUuid: string,
    @Body() updateNameDto: UpdateNameDto,
  ) {
    try {
      const userData = await this.userService.updateNameByUserUuid(
        userUuid,
        updateNameDto,
      );
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'name updated successfully',
        data: userData,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
